/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#ifndef CAOSDB_PROTOBUF_HELPER_H
#define CAOSDB_PROTOBUF_HELPER_H

#include "caosdb/status_code.h"
#include <google/protobuf/arena.h>         // for Arena
#include <google/protobuf/extension_set.h> // for Arena

#define CAOSDB_DEBUG_MESSAGE_STRING(message, out)                                                  \
  std::string out;                                                                                 \
  {                                                                                                \
    google::protobuf::util::JsonOptions options;                                                   \
    google::protobuf::util::MessageToJsonString(message, &out, options);                           \
  }

namespace caosdb::utility {

using google::protobuf::Arena;

auto get_arena() -> Arena *;

template <typename P> class ProtoMessageWrapper {
public:
  ProtoMessageWrapper(const ProtoMessageWrapper &other) = default;
  inline auto CopyFrom(const ProtoMessageWrapper &other) noexcept -> StatusCode {
    this->wrapped->CopyFrom(*other.wrapped);
    return StatusCode::SUCCESS;
  }

protected:
  ProtoMessageWrapper() : ProtoMessageWrapper(Arena::CreateMessage<P>(get_arena())) {}
  ProtoMessageWrapper(P *wrapped) : wrapped(wrapped) {}
  P *wrapped;
};

} // namespace caosdb::utility
#endif
