#include "caosdb/transaction_handler.h"
#include "caosdb/logging.h"                            // for CAOSDB_LOG_TRACE
#include <boost/log/core/record.hpp>                   // for record
#include <boost/log/detail/attachable_sstream_buf.hpp> // for basic_ostring...
#include <boost/log/sources/record_ostream.hpp>        // for basic_record_...
#include <boost/preprocessor/seq/limits/enum_256.hpp>  // for BOOST_PP_SEQ_...
#include <boost/preprocessor/seq/limits/size_256.hpp>  // for BOOST_PP_SEQ_...
#include <exception>                                   // IWYU pragma: keep
// IWYU pragma: no_include <bits/exception.h>
#include <grpcpp/impl/codegen/async_unary_call.h> // for ClientAsyncRes...
#include <grpcpp/impl/codegen/completion_queue.h> // for CompletionQueue
#include <iosfwd>                                 // for streamsize

namespace caosdb::transaction {

EntityTransactionHandler::EntityTransactionHandler(HandlerTag tag,
                                                   EntityTransactionService::Stub *stub,
                                                   grpc::CompletionQueue *completion_queue,
                                                   MultiTransactionRequest *request,
                                                   MultiTransactionResponse *response)
  : UnaryRpcHandler(completion_queue), tag_(tag), stub_(stub), request_(request),
    response_(response) {}

void EntityTransactionHandler::handleNewCallState() {
  CAOSDB_LOG_TRACE(logger_name) << "Enter EntityTransactionHandler::handleNewCallState with "
                                   "CompletionQueue "
                                << completion_queue;

  rpc_ = stub_->PrepareAsyncMultiTransaction(&call_context, *request_, completion_queue);

  state_ = CallState::CallComplete;
  rpc_->StartCall();
  rpc_->Finish(response_, &status_, tag_);

  CAOSDB_LOG_TRACE(logger_name) << "Leave EntityTransactionHandler::handleNewCallState";
}

} // namespace caosdb::transaction
