/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Daniel Hornung <d.hornung@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */
#include "caosdb/data_type.h"
#include "caosdb/entity.h"
#include "caosdb/utility.h"
#include <algorithm>

namespace caosdb::utility {

using caosdb::entity::AtomicDataType;
using caosdb::entity::Importance;
using caosdb::entity::Role;

// using emap = std::map<int, std::string>;          // enum mapping

// Enum helper template specializations //////////////////////////////////////
template <> auto getEnumNameFromValue<Importance>(Importance v) -> std::string {
  auto result = caosdb::entity::importance_names.at(v);
  return result;
}

template <> auto getEnumNameFromValue<Role>(Role v) -> std::string {
  auto result = caosdb::entity::role_names.at(v);
  return result;
}

template <> auto getEnumNameFromValue<AtomicDataType>(AtomicDataType v) -> std::string {
  auto result = caosdb::entity::atomicdatatype_names.at(v);
  return result;
}

template <> auto getEnumValueFromName<Importance>(const std::string &name) -> Importance {
  // TODO (dh): Why does this compile?
  // if (caosdb::entity::importance_names.begin()->second == name) {}
  // std::for_each(caosdb::entity::importance_names.begin(),
  //               caosdb::entity::importance_names.end(),
  //               [](const auto &entry){});
  // TODO (dh): Whereas this does not?
  // auto result = std::find(caosdb::entity::importance_names.cbegin(),
  //                         caosdb::entity::importance_names.cend(),
  //                         [name](const auto& entry){ return entry.second == name; });
  // Workaround: plaint old iteration:
  for (auto const &entry : caosdb::entity::importance_names) {
    if (entry.second == name) {
      return entry.first;
    }
  }
  throw std::out_of_range(std::string("Could not find enum value for string '") + name + "'.");
}

template <> auto getEnumValueFromName<AtomicDataType>(const std::string &name) -> AtomicDataType {
  for (auto const &entry : caosdb::entity::atomicdatatype_names) {
    if (entry.second == name) {
      return entry.first;
    }
  }
  throw std::out_of_range(std::string("Could not find enum value for string '") + name + "'.");
}

template <> auto getEnumValueFromName<Role>(const std::string &name) -> Role {
  for (auto const &entry : caosdb::entity::role_names) {
    if (entry.second == name) {
      return entry.first;
    }
  }
  throw std::out_of_range(std::string("Could not find enum value for string '") + name + "'.");
}

// End of template specialization /////////////////////////////////////////////

} // namespace caosdb::utility
