/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

// A simple caosdb client
#include "caosdb/connection.h"  // for Connection, ConnectionManager
#include "caosdb/constants.h"   // for LIBCAOSDB_VERSION_MINOR, LIBCAOSDB_V...
#include "caosdb/entity.h"      // for Entity
#include "caosdb/exceptions.h"  // for ConfigurationError
#include "caosdb/info.h"        // for VersionInfo
#include "caosdb/transaction.h" // for Transaction, UniqueResult, ResultSet
#include <iostream>             // for operator<<, basic_ostream, basic_ost...
#include <memory>               // for unique_ptr, allocator, __shared_ptr_...
#include <string>               // for operator<<, char_traits

auto main() -> int {

  std::cout << "CaosDB C++ client (libcaosdb " << caosdb::LIBCAOSDB_VERSION_MINOR << "."
            << caosdb::LIBCAOSDB_VERSION_MINOR << "." << caosdb::LIBCAOSDB_VERSION_PATCH << ")\n"
            << "We don't miss the H of caos.\n"
            << std::endl;

  try {
    const auto &connection = caosdb::connection::ConnectionManager::GetDefaultConnection();

    connection->RetrieveVersionInfoNoExceptions();
    // get version info of the server
    const auto &v_info = connection->GetVersionInfo();
    std::cout << "Server Version: " << v_info->GetMajor() << "." << v_info->GetMinor() << "."
              << v_info->GetPatch() << "-" << v_info->GetPreRelease() << "-" << v_info->GetBuild()
              << std::endl;

    // retrieve an entity
    auto transaction(connection->CreateTransaction());
    transaction->RetrieveById("20");
    transaction->Execute();
    const auto &result_set =
      dynamic_cast<const caosdb::transaction::UniqueResult &>(transaction->GetResultSet());

    // print description
    std::cout << "Entity Description: " << result_set.GetEntity().GetDescription() << std::endl;

    return 0;
  } catch (const caosdb::exceptions::ConfigurationError &exc) {
    std::cout << "ConfigurationError: " << exc.what() << std::endl;
    return exc.GetCode();
  }
}
